#!/usr/bin/jgawk -f
#
#  A tiny filter for Mailing List written in GNU AWK
#  Copyright (C) 1998 Mio Software lab. - Nishiyama, Naoki
#
#  Usage : (in alias) (j)gawk -v MLNAME=".." DOMAIN=".." -f aml.awk
#
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

BEGIN {
	VERSION   = "Mio's aml [Version 0.11 Alpha]"

	# Site Specific Setting
	MAILER     = "/usr/lib/sendmail"          # sendmail program
	SLEEP      = "/bin/sleep"
	REMOVE     = "/bin/rm"
	LOCKFILE   = "/tmp/.pidlock-" MLNAME

#	These option must be specified at Command Line...
#	MLNAME     = "TEST-ML"
#	DOMAIN     = "@mio.rim.or.jp"

	if (MLNAME == "" || DOMAIN == "") {
		print "aml : MLNAME and DOMAIN must be specified." > "/dev/stderr"
		exit -1
	}

	if (HEADER == "") HEADER = MLNAME
	REPLYTO    = MLNAME DOMAIN	          # Reply-To:  Header
        ERRORSTO   = MLNAME "-request" DOMAIN     # Errors-To: Header

	if (MLLIBDIR == "") MLLIBDIR = "/usr/lib/mail-list/"
	MEMBERFILE = MLLIBDIR MLNAME ".rec"       # ML Member list file
	SEQFILE    = MLLIBDIR MLNAME ".seq"       # ML Sequence number file

	#
	# Creating SENDMAIL Command Line..
	#
	while(getline <MEMBERFILE) {
		if (substr($0,1,1) != "#") MAILER = MAILER " " $1
	}

	getline MYPID < "/dev/pid"
}

#
# Bogus? Mutex System for ML Counter
#
function addmypid(fname,key,   okpid) {
	do {
		okpid = 0
		close(fname)
		getline okpid < fname
		if (okpid != 0) system(SLEEP " 1") ;
	} while(okpid != 0) ;
	close(fname)
	print key >> fname	# This output must be ATOMIC ... really?
	close(fname)
	getline okpid < fname
	return okpid
}

function getmutex(fname,key) {
	while(addmypid(fname,key) != key) ;
}

function relmutex(fname) {
	system(REMOVE " " fname)
}

#
# Delete These Lines of Header
#
/^Reply-To:/ || 
/^Sender:/   || 
/^Errors-To:/ {
	if (NOHEADER == 0) {
		DELNEXT = 1	# for Multiline header
		next
	}
}

#
# Delete Multiline Header
#
!/^[a-zA-Z0-9\-]+:/ {
	if (DELNEXT == 1 && NOHEADER == 0) next
}

#
# Adding Reply-To: Header and etc. (Before From:)
#
/^From:/{
	if (NOHEADER == 0) {
		print "X-ML-System: " VERSION  | MAILER
		print "Reply-To: "    REPLYTO  | MAILER
		print "Errors-To: "   ERRORSTO | MAILER
		print | MAILER
		DELNEXT = 0
		next
	}
}

#
# Adding [ML-NAME: xxxx] to Subject 
#
/^Subject:/{
	if (NOHEADER == 0) {
		sub("^Subject: ","")
		sub("\[[a-zA-Z\-]+: [0-9]+\] ","")
		sub("^(R[Ee]:( )*)+","Re: ")
		getmutex(LOCKFILE,MYPID)	# Critical Section
		getline NUMBER < SEQFILE
		print   NUMBER+1 > SEQFILE
		relmutex(LOCKFILE)		# End of Critical Section
		print "Subject: [" HEADER ": "  NUMBER+1 "] " $0 | MAILER
		DELNEXT = 0
		next
	}		
}

#
# Header End Detect
#
/^$/ {
	NOHEADER = 1
}

{
	DELNEXT = 0
	print | MAILER
}

END {
	close(MAILER)
}
